#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("Poly10Grammar", "Represents a grammar for modeling with fixed structures for the voest B1 MwDru target variable.")]
  public class Poly10Grammar : SymbolicExpressionGrammar, ISymbolicDataAnalysisGrammar {

    [StorableConstructor]
    protected Poly10Grammar(bool deserializing) : base(deserializing) { }
    protected Poly10Grammar(Poly10Grammar original, Cloner cloner) : base(original, cloner) { }
    public Poly10Grammar()
      : base(ItemAttribute.GetName(typeof(Poly10Grammar)), ItemAttribute.GetDescription(typeof(TypeCoherentExpressionGrammar))) {
      Initialize();
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new Poly10Grammar(this, cloner);
    }

    private void Initialize() {
      var add = new Addition();
      var sub = new Subtraction();
      var mul = new Multiplication();
      var div = new Division();

      var constant = new Constant();
      var variableSymbol = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();

      var fixedAdd1 = new Addition();
      fixedAdd1.Name = "fixed Addition1";
      fixedAdd1.Fixed = true;

      var fixedAdd2 = new Addition();
      fixedAdd2.Name = "fixed Addition2";
      fixedAdd2.Fixed = true;

      var fixedAdd3 = new Addition();
      fixedAdd3.Name = "fixed Addition3";
      fixedAdd3.Fixed = true;

      var fixedMul1 = new Multiplication();
      fixedMul1.Name = "fixed Multiplication1";
      fixedMul1.Fixed = true;

      var fixedMul2 = new Multiplication();
      fixedMul2.Name = "fixed Multiplication2";
      fixedMul2.Fixed = true;


      #region fixed variables
      var x1 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x1.Name = "fixed X1";
      x1.VariableNames = new List<string>() { "x1" };
      x1.Fixed = true;


      var x2 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x2.Name = "fixed X2";
      x2.VariableNames = new List<string>() { "x2" };
      x2.Fixed = true;

      var x3 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x3.Name = "fixed X3";
      x3.VariableNames = new List<string>() { "x3" };
      x3.Fixed = true;

      var x4 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x4.Name = "fixed X4";
      x4.VariableNames = new List<string>() { "x4" };
      x4.Fixed = true;

      var x5 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x5.Name = "fixed X5";
      x5.VariableNames = new List<string>() { "x5" };
      x5.Fixed = true;

      var x6 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x6.Name = "fixed X6";
      x6.VariableNames = new List<string>() { "x6" };
      x6.Fixed = true;

      var x7 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x7.Name = "fixed X7";
      x7.VariableNames = new List<string>() { "x7" };
      x7.Fixed = true;

      var x8 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x8.Name = "fixed X8";
      x8.VariableNames = new List<string>() { "x8" };
      x8.Fixed = true;

      var x9 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x9.Name = "fixed X9";
      x9.VariableNames = new List<string>() { "x9" };
      x9.Fixed = true;

      var x10 = new HeuristicLab.Problems.DataAnalysis.Symbolic.Variable();
      x10.Name = "fixed X10";
      x10.VariableNames = new List<string>() { "x10" };
      x10.Fixed = true;
      #endregion

      var fixedTerminalSymbols = new List<HeuristicLab.Problems.DataAnalysis.Symbolic.Variable>() { x1, x2, x3, x4, x5, x6, x7, x8, x9, x10 };
      var allSymbols = new List<Symbol>() 
        { add, sub, mul, div, constant, variableSymbol,
          fixedAdd1,fixedMul1,fixedMul2}.Union(fixedTerminalSymbols);

      var binaryFunctionSymbols = new List<Symbol>() { add, sub, mul, div };
      var terminalSymbols = new List<Symbol>() { variableSymbol, constant };
      var realValuedSymbols = binaryFunctionSymbols.Concat(terminalSymbols);

      foreach (var symb in allSymbols)
        AddSymbol(symb);
      foreach (var terminalSymbol in terminalSymbols.Union(fixedTerminalSymbols)) {
        SetSubtreeCount(terminalSymbol, 0, 0);
      }
      foreach (var binaryFun in binaryFunctionSymbols) {
        SetSubtreeCount(binaryFun, 2, 2);
      }
      SetSubtreeCount(fixedAdd1, 3, 3);
      SetSubtreeCount(fixedMul1, 3, 3);
      SetSubtreeCount(fixedMul2, 3, 3);

      AddAllowedChildSymbol(StartSymbol, fixedAdd1);
      foreach (var symb in realValuedSymbols) {
        AddAllowedChildSymbol(fixedAdd1, symb, 0);
      }

      AddAllowedChildSymbol(fixedAdd1, fixedMul1, 1);
      AddAllowedChildSymbol(fixedAdd1, fixedMul2, 2);


      AddAllowedChildSymbol(fixedMul1, x1, 0);
      AddAllowedChildSymbol(fixedMul1, x7, 1);
      AddAllowedChildSymbol(fixedMul1, x9, 2);

      AddAllowedChildSymbol(fixedMul2, x3, 0);
      AddAllowedChildSymbol(fixedMul2, x6, 1);
      AddAllowedChildSymbol(fixedMul2, x10, 2);

      foreach (var symb in binaryFunctionSymbols) {
        foreach (var childSymb in realValuedSymbols) {
          AddAllowedChildSymbol(symb, childSymb);
        }
      }
    }
  }
}
